package com.mortals.xhx.module.site.service.impl;
import com.mortals.framework.common.Rest;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.model.Result;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.common.utils.BeanUtil;
import com.mortals.xhx.module.matter.model.*;
import com.mortals.xhx.module.matter.service.MatterDatumService;
import com.mortals.xhx.module.single.model.SingleMatterEntity;
import com.mortals.xhx.module.single.model.SingleMatterQuery;
import com.mortals.xhx.module.single.service.SingleMatterService;
import com.mortals.xhx.module.site.model.SiteEntity;
import com.mortals.xhx.module.site.model.SiteMatterQuery;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.xhx.module.site.dao.SiteMatterDao;
import com.mortals.xhx.module.site.model.SiteMatterEntity;
import com.mortals.xhx.module.site.service.SiteMatterService;
import org.springframework.util.ObjectUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
* SiteMatterService
* 站点事项 service实现
*
* @author zxfei
* @date 2023-03-10
*/
@Service("siteMatterService")
public class SiteMatterServiceImpl extends AbstractCRUDServiceImpl<SiteMatterDao, SiteMatterEntity, Long> implements SiteMatterService {

    @Autowired
    private MatterDatumService matterDatumService;
    @Autowired
    private SingleMatterService singleMatterService;

    @Override
    protected void findAfter(SiteMatterEntity params, PageInfo pageInfo, Context context, List<SiteMatterEntity> list) throws AppException {
        list.stream().peek(item->{
            List<MatterDatumEntity> matterDatumFileEntities = matterDatumService.find(new MatterDatumQuery().matterId(item.getMatterId()));
            if(CollectionUtils.isNotEmpty(matterDatumFileEntities)) {
                item.setDatumCount(matterDatumFileEntities.size());
            }else {
                item.setDatumCount(0);
            }
        }).count();
        super.findAfter(params, context, list);
    }

    @Override
    protected void findAfter(SiteMatterEntity params, Context context, List<SiteMatterEntity> list) throws AppException {
        list.stream().peek(item->{
            List<MatterDatumEntity> matterDatumFileEntities = matterDatumService.find(new MatterDatumQuery().matterId(item.getMatterId()));
            if(CollectionUtils.isNotEmpty(matterDatumFileEntities)) {
                item.setDatumCount(matterDatumFileEntities.size());
            }else {
                item.setDatumCount(0);
            }
        }).count();
        super.findAfter(params, context, list);
    }

    @Override
    public void addViewsCount(Long id) {
        Map<String,Object> data = new HashMap<>();
        data.put("viewsCountIncrement",1);
        Map<String,Object> condition = new HashMap<>();
        condition.put("id",id);
        dao.update(data,condition);
    }

    @Override
    public Rest<String> addToSingle(String siteMatterIds, Context context) {
        List<Long> matterIdList = Arrays.asList(siteMatterIds.split(",")).stream().map(Long::parseLong).collect(Collectors.toList());
        SiteMatterQuery matterQuery = new SiteMatterQuery();
        matterQuery.setIdList(matterIdList);
        List<SiteMatterEntity> matterEntities = this.find(matterQuery);
        int success = 0;
        int fail = 0;
        List<SingleMatterEntity> singelMatterEntities = new ArrayList<>();

        for (SiteMatterEntity matterEntity : matterEntities) {
            SingleMatterEntity singelMatterEntity = updateOrSave(matterEntity, context);
            if (!ObjectUtils.isEmpty(singelMatterEntity)) {
                singelMatterEntities.add(singelMatterEntity);
                success++;
            } else {
                fail++;
            }
        }
        if (!ObjectUtils.isEmpty(singelMatterEntities)) {
            singleMatterService.save(singelMatterEntities, context);
        }
        String msg = "当前加入事项已存在！";
        if (matterIdList.size() == 1) {
            if (success > 0) {
                msg = "加入事项成功！";
            }
        } else if (matterIdList.size() > 1) {
            if (success > 0 && fail == 0) {
                msg = String.format("加入事项成功%d条!", success);
            } else if (success > 0 && fail > 0) {
                msg = String.format("加入事项成功%d条，重复加入事项%d条！", success, fail);
            } else if (success == 0 && fail > 0) {
                msg = String.format("重复加入事项%d条！", fail);
            }
        }
        return Rest.ok(msg);
    }

    private SingleMatterEntity updateOrSave(SiteMatterEntity item, Context context) {
        SingleMatterEntity singleMatterEntity = singleMatterService.selectOne(new SingleMatterQuery().siteMatterId(item.getId()));

        if (ObjectUtils.isEmpty(singleMatterEntity)) {
            SingleMatterEntity singleEntity = BeanUtil.covert(item,SingleMatterEntity.class);
            singleEntity.setCreateTime(new Date());
            if(context!=null&&context.getUser()!=null) {
                singleEntity.setCreateUserId(context.getUser().getId());
            }
            singleEntity.setSiteMatterId(item.getId());
            singleEntity.setViewsCount(0l);
            singleEntity.setIsConvert(0);
            return singleEntity;
        }else {
        return null;
        }
    }

    @Override
    public Result<SiteMatterEntity> findSubList(SiteMatterEntity matterQuery, PageInfo pageInfo, Context context) throws AppException {
        return this.dao.getSubList(matterQuery, pageInfo);
    }
}